@@ -660,6 +660,28 @@ def JacobianBody(Blist, thetalist):
                                          * -thetalist[i + 1])))
         Jb[:, i] = np.dot(Adjoint(T), np.array(Blist)[:, i])
     return Jb
+def DerivativeJacobianBody(Blist,thetalist):
+    n = len(thetalist)
+    dvecJb = np.zeros([6*n,n])
+    Jb = JacobianBody(Blist,thetalist);
+    for i in range(0,n):
+            for j in range(0,n):    
+                if(i>j):
+                    Jb_j = Jb[:,j]
+                    Jb_i = Jb[:,i]      
+                    dvecJb[6*j:6*j+6,i] = ad(Jb_j)*Jb_i              
+    return dvecJb
+def DerivativeJacobianSpace(Slist,thetalist):
+    n = len(thetalist)
+    dvecJs = np.zeros([6*n,n])
+    Js = JacobianSpace(Slist,thetalist);
+    for i in range(0,n):
+            for j in range(0,n):    
+                if(i>j):
+                    Js_j = Js[:,j]
+                    Js_i = Js[:,i]      
+                    dvecJs[6*j:6*j+6,i] = ad(Js_j)*Js_i              
+    return dvecJs    
 def AnalyticJacobianBody(M,Blist, thetalist):
     Jb =  JacobianBody(Blist, thetalist)
     Tsb = FKinBody(M,Blist, thetalist)
@@ -1545,7 +1567,21 @@ def CubicTimeScaling(Tf, t):
         0.216
     """
     return 3 * (1.0 * t / Tf) ** 2 - 2 * (1.0 * t / Tf) ** 3
+def CubicTimeScalingDot(Tf, t):
+    """Computes sdot(t) for a cubic time scaling
 
+    :param Tf: Total time of the motion in seconds from rest to rest
+    :param t: The current time t satisfying 0 < t < Tf
+    :return: The path parameter sdot(t) corresponding to a third-order
+             polynomial motion that begins and ends at zero velocity
+
+    Example Input:
+        Tf = 2
+        t = 0.6
+    Output:
+        0.216
+    """
+    return 2*3 * (1.0 * t / Tf)/ Tf  - 2 *3* (1.0 * t / Tf) ** 2/ Tf
 def QuinticTimeScaling(Tf, t):
     """Computes s(t) for a quintic time scaling
 
@@ -1563,7 +1599,23 @@ def QuinticTimeScaling(Tf, t):
     """
     return 10 * (1.0 * t / Tf) ** 3 - 15 * (1.0 * t / Tf) ** 4 \
            + 6 * (1.0 * t / Tf) ** 5
+def QuinticTimeScalingDot(Tf, t):
+    """Computes sdot(t) for a quintic time scaling
 
+    :param Tf: Total time of the motion in seconds from rest to rest
+    :param t: The current time t satisfying 0 < t < Tf
+    :return: The path parameter sdot(t) corresponding to a fifth-order
+             polynomial motion that begins and ends at zero velocity and zero
+             acceleration
+
+    Example Input:
+        Tf = 2
+        t = 0.6
+    Output:
+        0.16308
+    """
+    return 10*3 * (1.0 * t / Tf) ** 2/ Tf - 15 * 4* (1.0 * t / Tf) ** 3/ Tf \
+           + 6 *5 (1.0 * t / Tf) ** 4/ Tf
 def JointTrajectory(thetastart, thetaend, Tf, N, method):
     """Computes a straight-line trajectory in joint space
 
@@ -1664,7 +1716,71 @@ def ScrewTrajectory(Xstart, Xend, Tf, N, method):
         = np.dot(Xstart, MatrixExp6(MatrixLog6(np.dot(TransInv(Xstart), \
                                                       Xend)) * s))
     return traj
+def CartesianVelTrajectory(Xstart, Xend, Tf, N, method):
+    """Computes a trajectory as a list of N SE(3) matrices corresponding to
+    the origin of the end-effector frame following a straight line
 
+    :param Xstart: The initial end-effector configuration
+    :param Xend: The final end-effector configuration
+    :param Tf: Total time of the motion in seconds from rest to rest
+    :param N: The number of points N > 1 (Start and stop) in the discrete
+              representation of the trajectory
+    :param method: The time-scaling method, where 3 indicates cubic (third-
+                   order polynomial) time scaling and 5 indicates quintic
+                   (fifth-order polynomial) time scaling
+    :return: The discretized trajectory as a list of N matrices in SE(3)
+             separated in time by Tf/(N-1). The first in the list is Xstart
+             and the Nth is Xend
+    This function is similar to ScrewTrajectory, except the origin of the
+    end-effector frame follows a straight line, decoupled from the rotational
+    motion.
+
+    Example Input:
+        Xstart = np.array([[1, 0, 0, 1],
+                           [0, 1, 0, 0],
+                           [0, 0, 1, 1],
+                           [0, 0, 0, 1]])
+        Xend = np.array([[0, 0, 1, 0.1],
+                         [1, 0, 0,   0],
+                         [0, 1, 0, 4.1],
+                         [0, 0, 0,   1]])
+        Tf = 5
+        N = 4
+        method = 5
+    Output:
+        [np.array([[1, 0, 0, 1]
+                   [0, 1, 0, 0]
+                   [0, 0, 1, 1]
+                   [0, 0, 0, 1]]),
+         np.array([[ 0.937, -0.214,  0.277, 0.811]
+                   [ 0.277,  0.937, -0.214,     0]
+                   [-0.214,  0.277,  0.937, 1.651]
+                   [     0,      0,      0,     1]]),
+         np.array([[ 0.277, -0.214,  0.937, 0.289]
+                   [ 0.937,  0.277, -0.214,     0]
+                   [-0.214,  0.937,  0.277, 3.449]
+                   [     0,      0,      0,     1]]),
+         np.array([[0, 0, 1, 0.1]
+                   [1, 0, 0,   0]
+                   [0, 1, 0, 4.1]
+                   [0, 0, 0,   1]])]
+    """
+    N = int(N)
+    timegap = Tf / (N - 1.0)
+    traj = [[None]] * N
+    Rstart, pstart = TransToRp(Xstart)
+    Rend, pend = TransToRp(Xend)
+    for i in range(N):
+        if method == 3:
+            sdot = CubicTimeScalingDot(Tf, timegap * i)
+        else:
+            sdot = QuinticTimeScalingDot(Tf, timegap * i)
+        traj[i] \
+        = np.r_[np.c_[np.dot(Rstart, \
+        np.dot(MatrixLog3(np.dot(np.array(Rstart).T,Rend)),MatrixExp3(MatrixLog3(np.dot(np.array(Rstart).T,Rend))*s) * sdot)), \
+                   sdot * np.array(pend)  - sdot* np.array(pstart)], \
+                   [[0, 0, 0, 1]]]
+    return traj
 def CartesianTrajectory(Xstart, Xend, Tf, N, method):
     """Computes a trajectory as a list of N SE(3) matrices corresponding to
     the origin of the end-effector frame following a straight line
